/* THIS FILE IS AUTOGENERATED - DO NOT EDIT */

#ifndef mozilla_dom_MobileMessageManagerBinding_h__
#define mozilla_dom_MobileMessageManagerBinding_h__

#include "mozilla/ErrorResult.h"
#include "mozilla/dom/BindingDeclarations.h"
#include "mozilla/dom/DOMJSClass.h"
#include "mozilla/dom/DOMJSProxyHandler.h"
#include "mozilla/dom/Nullable.h"

namespace mozilla {
namespace dom {

struct MmsAttachment;

} // namespace dom
} // namespace mozilla

namespace mozilla {
namespace dom {

struct MmsParameters : public MainThreadDictionaryBase
{
  Optional<Sequence<MmsAttachment > > mAttachments;
  Optional<Sequence<nsString > > mReceivers;
  nsString mSmil;
  nsString mSubject;

private:
  static bool initedIds;
  static jsid attachments_id;
  static jsid receivers_id;
  static jsid smil_id;
  static jsid subject_id;

  MmsParameters(const MmsParameters&) MOZ_DELETE;
  void operator=(const MmsParameters) MOZ_DELETE;

public:
  inline MmsParameters()
  {
  }

  bool
  Init(JSContext* cx, JS::Handle<JS::Value> val, const char* sourceDescription = "Value");

  bool
  Init(const nsAString& aJSON);

  bool
  ToObject(JSContext* cx, JS::Handle<JSObject*> parentObject, JS::MutableHandle<JS::Value> rval) const;

  void
  TraceDictionary(JSTracer* trc);

private:
  static bool
  InitIds(JSContext* cx);
};
struct MmsParametersInitializer : public MmsParameters
{
  inline MmsParametersInitializer()
  {
    // Safe to pass a null context if we pass a null value
    Init(nullptr, JS::NullHandleValue);
  }
};


} // namespace dom
} // namespace mozilla

#endif // mozilla_dom_MobileMessageManagerBinding_h__
