/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#ifndef NSEDITORCID_H___

#define NS_EDITOR_CID \
{/* {D3DE3431-8A75-11d2-918C-0080C8E44DB5}*/ \
0xd3de3431, 0x8a75, 0x11d2, \
{ 0x91, 0x8c, 0x0, 0x80, 0xc8, 0xe4, 0x4d, 0xb5 } }

#define NS_TEXTEDITOR_CID \
{/* {e197cc01-cfe1-11d4-8eb0-87ae406dfd3f}*/ \
0xe197cc01, 0xcfe1, 0x11d4, \
{ 0x8e, 0xb0, 0x87, 0xae, 0x40, 0x6d, 0xfd, 0x3f } }

#define NS_HTMLEDITOR_CID \
{/* {ed0244e0-c144-11d2-8f4c-006008159b0c}*/ \
0xed0244e0, 0xc144, 0x11d2, \
{ 0x8f, 0x4c, 0x0, 0x60, 0x08, 0x15, 0x9b, 0x0c } }

#endif //NSEDITORCID_H___



