/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

[scriptable, uuid(9e631b1d-7a07-4925-a47e-697c6edaf408)]
interface nsIIMEPicker : nsISupports
{
 /**
  * Show IME Picker dialog.
  */
  void show();
};
