/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "domstubs.idl"

[scriptable, uuid(2cb53a8a-d2f4-4ddf-874f-3bc2d595c41a)]
interface nsIDOMLoadStatus : nsISupports
{
  readonly attribute nsIDOMNode source;
  readonly attribute DOMString uri;
  readonly attribute long totalSize;
  readonly attribute long loadedSize;
  readonly attribute unsigned short readyState;
  readonly attribute unsigned short status;

  const unsigned short UNINITIALIZED = 0;
  const unsigned short REQUESTED     = 1;
  const unsigned short RECEIVING     = 2;
  const unsigned short LOADED        = 3;
};
